-- Lesson: 	Coupled systems - Prey-predator model. 
--			.
-- Author:	Tiago Carneiro

-----------------------------------------------------------------
-- PARAMETER
-----------------------------------------------------------------

BIRTH_PREY_RATE 			 = 0.2
PREDATION_RATE				 = 0.01 -- death prey rate
BIRTH_PREDATOR_PER_PREY_RATE = 0.01 -- birth predator rate
DEATH_PREDATOR_RATE			 = 0.1  -- death predator rate
DT							 = 0.03125

-----------------------------------------------------------------
-- MODEL
-----------------------------------------------------------------

function preysChange( preysPast, predatorsPast )
	return preysPast*BIRTH_PREY_RATE - preysPast*predatorsPast*PREDATION_RATE
end 


function predatorsChange( preysPast, predatorsPast )
	return predatorsPast*preysPast*BIRTH_PREDATOR_PER_PREY_RATE - predatorsPast*DEATH_PREDATOR_RATE
end 

ag = Agent{
	preys  		= 100,
	predators	= 10,
	execute		= function( self )
		local  dPreys 		= preysChange( self.preys, self.predators ) * DT
		local  dPredators 	= predatorsChange( self.preys, self.predators ) * DT
		self.preys 			= self.preys + dPreys 
		self.predators 		= self.predators + dPredators 
	end
}

-----------------------------------------------------------------
-- SIMULATION
-----------------------------------------------------------------

obs = Observer{subject = ag, type = "chart", attributes = {"preys", "predators"} }

for t = 1, 200, DT do

	ag:execute();
		
	ag:notify(t)
	
end